@@ -160,7 +160,7 @@ class KernelClient(ConnectionFileMixin):
 
         # Wait for kernel info reply on shell channel
         while True:
-            self._kernel_info()
+            self.kernel_info()
             try:
                 msg = await self.shell_channel.get_msg(timeout=1)
             except Empty:
@@ -386,10 +386,9 @@ class KernelClient(ConnectionFileMixin):
             # We don't have access to the KernelManager,
             # so we use the heartbeat.
             return self._hb_channel.is_beating()
-        else:
-            # no heartbeat and not local, we can't tell if it's running,
-            # so naively return True
-            return True
+        # no heartbeat and not local, we can't tell if it's running,
+        # so naively return True
+        return True
 
     async def _async_execute_interactive(
         self,
@@ -463,7 +462,7 @@ class KernelClient(ConnectionFileMixin):
             allow_stdin = self.allow_stdin
         if allow_stdin and not self.stdin_channel.is_alive():
             raise RuntimeError("stdin channel must be running to allow input")
-        msg_id = self._execute(
+        msg_id = self.execute(
             code,
             silent=silent,
             store_history=store_history,
@@ -541,7 +540,7 @@ class KernelClient(ConnectionFileMixin):
         return await self._async_recv_reply(msg_id, timeout=timeout)
 
     # Methods to send specific messages on channels
-    def _execute(
+    def execute(
         self,
         code: str,
         silent: bool = False,
@@ -608,7 +607,7 @@ class KernelClient(ConnectionFileMixin):
         self.shell_channel.send(msg)
         return msg["header"]["msg_id"]
 
-    def _complete(self, code: str, cursor_pos: t.Optional[int] = None) -> str:
+    def complete(self, code: str, cursor_pos: t.Optional[int] = None) -> str:
         """Tab complete text in the kernel's namespace.
 
         Parameters
@@ -631,7 +630,7 @@ class KernelClient(ConnectionFileMixin):
         self.shell_channel.send(msg)
         return msg["header"]["msg_id"]
 
-    def _inspect(self, code: str, cursor_pos: t.Optional[int] = None, detail_level: int = 0) -> str:
+    def inspect(self, code: str, cursor_pos: t.Optional[int] = None, detail_level: int = 0) -> str:
         """Get metadata information about an object in the kernel's namespace.
 
         It is up to the kernel to determine the appropriate object to inspect.
@@ -662,7 +661,7 @@ class KernelClient(ConnectionFileMixin):
         self.shell_channel.send(msg)
         return msg["header"]["msg_id"]
 
-    def _history(
+    def history(
         self,
         raw: bool = True,
         output: bool = False,
@@ -708,7 +707,7 @@ class KernelClient(ConnectionFileMixin):
         self.shell_channel.send(msg)
         return msg["header"]["msg_id"]
 
-    def _kernel_info(self) -> str:
+    def kernel_info(self) -> str:
         """Request kernel info
 
         Returns
@@ -719,7 +718,7 @@ class KernelClient(ConnectionFileMixin):
         self.shell_channel.send(msg)
         return msg["header"]["msg_id"]
 
-    def _comm_info(self, target_name: t.Optional[str] = None) -> str:
+    def comm_info(self, target_name: t.Optional[str] = None) -> str:
         """Request comm info
 
         Returns
@@ -760,7 +759,7 @@ class KernelClient(ConnectionFileMixin):
         msg = self.session.msg("input_reply", content)
         self.stdin_channel.send(msg)
 
-    def _shutdown(self, restart: bool = False) -> str:
+    def shutdown(self, restart: bool = False) -> str:
         """Request an immediate kernel shutdown on the control channel.
 
         Upon receipt of the (empty) reply, client code can safely assume that
