@@ -19,6 +19,7 @@ from typing import cast
 from typing import Dict
 from typing import List
 from typing import Optional
+from typing import Set
 from typing import Tuple
 from typing import Union
 
@@ -29,29 +30,34 @@ from jupyter_core.paths import secure_write
 from traitlets import Bool  # type: ignore
 from traitlets import CaselessStrEnum
 from traitlets import Instance
+from traitlets import Int
 from traitlets import Integer
 from traitlets import observe
 from traitlets import Type
 from traitlets import Unicode
 from traitlets.config import LoggingConfigurable  # type: ignore
+from traitlets.config import SingletonConfigurable
 
 from .localinterfaces import localhost
 from .utils import _filefind
 
+# Define custom type for kernel connection info
+KernelConnectionInfo = Dict[str, Union[int, str, bytes]]
+
 
 def write_connection_file(
     fname: Optional[str] = None,
-    shell_port: int = 0,
-    iopub_port: int = 0,
-    stdin_port: int = 0,
-    hb_port: int = 0,
-    control_port: int = 0,
+    shell_port: Union[Integer, Int, int] = 0,
+    iopub_port: Union[Integer, Int, int] = 0,
+    stdin_port: Union[Integer, Int, int] = 0,
+    hb_port: Union[Integer, Int, int] = 0,
+    control_port: Union[Integer, Int, int] = 0,
     ip: str = "",
     key: bytes = b"",
     transport: str = "tcp",
     signature_scheme: str = "hmac-sha256",
     kernel_name: str = "",
-) -> Tuple[str, Dict[str, Union[int, str]]]:
+) -> Tuple[str, KernelConnectionInfo]:
     """Generates a JSON config file, including the selection of random ports.
 
     Parameters
@@ -139,7 +145,7 @@ def write_connection_file(
     if hb_port <= 0:
         hb_port = ports.pop(0)
 
-    cfg: Dict[str, Union[int, str]] = dict(
+    cfg: KernelConnectionInfo = dict(
         shell_port=shell_port,
         iopub_port=iopub_port,
         stdin_port=stdin_port,
@@ -250,7 +256,7 @@ def find_connection_file(
 
 
 def tunnel_to_kernel(
-    connection_info: Union[str, Dict[str, Any]],
+    connection_info: Union[str, KernelConnectionInfo],
     sshserver: str,
     sshkey: Optional[str] = None,
 ) -> Tuple[Any, ...]:
@@ -398,7 +404,7 @@ class ConnectionFileMixin(LoggingConfigurable):
     # Connection and ipc file management
     # --------------------------------------------------------------------------
 
-    def get_connection_info(self, session: bool = False) -> Dict[str, Any]:
+    def get_connection_info(self, session: bool = False) -> KernelConnectionInfo:
         """Return the connection info as a dict
 
         Parameters
@@ -543,7 +549,7 @@ class ConnectionFileMixin(LoggingConfigurable):
             info = json.load(f)
         self.load_connection_info(info)
 
-    def load_connection_info(self, info: Dict[str, int]) -> None:
+    def load_connection_info(self, info: KernelConnectionInfo) -> None:
         """Load connection info from a dict containing connection info.
 
         Typically this data comes from a connection file
@@ -574,6 +580,19 @@ class ConnectionFileMixin(LoggingConfigurable):
         if "signature_scheme" in info:
             self.session.signature_scheme = info["signature_scheme"]
 
+    def _force_connection_info(self, info: KernelConnectionInfo) -> None:
+        """Unconditionally loads connection info from a dict containing connection info.
+
+        Overwrites connection info-based attributes, regardless of their current values
+        and writes this information to the connection file.
+        """
+        # Reset current ports to 0 and indicate file has not been written to enable override
+        self._connection_file_written = False
+        for name in port_names:
+            setattr(self, name, 0)
+        self.load_connection_info(info)
+        self.write_connection_file()
+
     # --------------------------------------------------------------------------
     # Creating connected sockets
     # --------------------------------------------------------------------------
@@ -627,8 +646,44 @@ class ConnectionFileMixin(LoggingConfigurable):
         return self._create_connected_socket("control", identity=identity)
 
 
+class LocalPortCache(SingletonConfigurable):
+    """
+    Used to keep track of local ports in order to prevent race conditions that
+    can occur between port acquisition and usage by the kernel.  All locally-
+    provisioned kernels should use this mechanism to limit the possibility of
+    race conditions.  Note that this does not preclude other applications from
+    acquiring a cached but unused port, thereby re-introducing the issue this
+    class is attempting to resolve (minimize).
+    See: https://github.com/jupyter/jupyter_client/issues/487
+    """
+
+    def __init__(self, **kwargs) -> None:
+        super().__init__(**kwargs)
+        self.currently_used_ports: Set[int] = set()
+
+    def find_available_port(self, ip: str) -> int:
+        while True:
+            tmp_sock = socket.socket()
+            tmp_sock.setsockopt(socket.SOL_SOCKET, socket.SO_LINGER, b"\0" * 8)
+            tmp_sock.bind((ip, 0))
+            port = tmp_sock.getsockname()[1]
+            tmp_sock.close()
+
+            # This is a workaround for https://github.com/jupyter/jupyter_client/issues/487
+            # We prevent two kernels to have the same ports.
+            if port not in self.currently_used_ports:
+                self.currently_used_ports.add(port)
+                return port
+
+    def return_port(self, port: int) -> None:
+        if port in self.currently_used_ports:  # Tolerate uncached ports
+            self.currently_used_ports.remove(port)
+
+
 __all__ = [
     "write_connection_file",
     "find_connection_file",
     "tunnel_to_kernel",
+    "KernelConnectionInfo",
+    "LocalPortCache",
 ]
