@@ -1,17 +1,11 @@
 """
 utils:
-- provides utility wrapeprs to run asynchronous functions in a blocking environment.
+- provides utility wrappers to run asynchronous functions in a blocking environment.
 - vendor functions from ipython_genutils that should be retired at some point.
 """
 import asyncio
 import inspect
 import os
-import sys
-
-import nest_asyncio  # type: ignore
-
-if os.name == "nt" and sys.version_info >= (3, 7):
-    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())  # type: ignore
 
 
 def run_sync(coro):
@@ -21,6 +15,8 @@ def run_sync(coro):
         except RuntimeError:
             loop = asyncio.new_event_loop()
             asyncio.set_event_loop(loop)
+        import nest_asyncio  # type: ignore
+
         nest_asyncio.apply(loop)
         return loop.run_until_complete(coro(*args, **kwargs))
 
